@@ -21,33 +21,31 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFI Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
  
 #include "mpfi.h"
 #include "mpfi-impl.h"
 
 /* Conversion of a mpfr which has an integer value into a mpz */
-void mpfi_mpz_exact_set_fr (mpz_ptr z, mpfr_srcptr x)
+/*
+static void mpfi_mpz_exact_set_fr (mpz_ptr z, mpfr_srcptr x)
 {
   mp_exp_t expo;
-  int sign;
 
-  sign = MPFR_SIGN(x);
   expo = mpfr_get_z_exp(z, x);
   if (expo >= 0)
     mpz_mul_2exp(z, z, expo);
   else
     mpz_fdiv_q_2exp(z, z, -expo);
-  if (sign < 0)
-    mpz_neg(z, z);
 }
+*/
 
 /* returns in quad the integer part of the division of x by Pi/2        */
 /* the result is exact                                                  */
 /* the returned value is the precision required to perform the division */
-mp_prec_t mpfi_quadrant (mpz_ptr quad, mpfr_srcptr x)
+static mp_prec_t mpfi_quadrant (mpz_ptr quad, mpfr_srcptr x)
 {
 /* Assumption: x is neither a NaN nor an Infinite */
   int ok=0;
@@ -79,7 +77,7 @@ mp_prec_t mpfi_quadrant (mpz_ptr quad, mpfr_srcptr x)
       }
     } while (ok != 0);
 
-    mpfi_mpz_exact_set_fr(quad, &(tmp->left));
+    mpfr_get_z(quad, &(tmp->left), GMP_RNDN); //exact
 
     mpfi_clear(two_over_pi);
     mpfi_clear(tmp);
@@ -89,7 +87,7 @@ mp_prec_t mpfi_quadrant (mpz_ptr quad, mpfr_srcptr x)
 
 /* compares z * Pi/2 - x and y where z is an integer (mpz)              */
 /* the result is exact                                                  */
-int mpfi_cmp_sym_pi (mpz_srcptr z, mpfr_srcptr x, mpfr_srcptr y, mp_prec_t prec_init)
+static int mpfi_cmp_sym_pi (mpz_srcptr z, mpfr_srcptr x, mpfr_srcptr y, mp_prec_t prec_init)
 {
 /* Assumption: x and y are neither NaN nor Infinite */
   mp_prec_t prec;
@@ -101,20 +99,28 @@ int mpfi_cmp_sym_pi (mpz_srcptr z, mpfr_srcptr x, mpfr_srcptr y, mp_prec_t prec_
   mpfi_init2(pi_over_two, prec);
   mpfi_init2(tmp, prec);
 
-  do {
-    mpfi_const_pi(pi_over_two);
-    mpfi_div_2exp(pi_over_two, pi_over_two, 1);
+  if(mpz_cmp_ui(z,0)==0) {
+    not_ok=0;
+    mpfi_set_prec(tmp, mpfr_get_prec(x));
+    mpfi_set_fr(tmp,x);
+    mpfi_neg(tmp, tmp);
+  }
+  else {
+    do {
+      mpfi_const_pi(pi_over_two);
+      mpfi_div_2exp(pi_over_two, pi_over_two, 1);
 
-    mpfi_mul_z(tmp, pi_over_two, z);
-    mpfi_sub_fr(tmp, tmp, x);
+      mpfi_mul_z(tmp, pi_over_two, z);
+      mpfi_sub_fr(tmp, tmp, x);
 
-    not_ok = mpfi_is_inside_fr(y, tmp);
-    if (not_ok) {
-      prec += BITS_PER_MP_LIMB;
-      mpfi_set_prec(pi_over_two, prec);
-      mpfi_set_prec(tmp, prec);
-    }
-  } while (not_ok);
+      not_ok = mpfi_is_inside_fr(y, tmp);
+      if (not_ok) {
+	prec += BITS_PER_MP_LIMB;
+	mpfi_set_prec(pi_over_two, prec);
+	mpfi_set_prec(tmp, prec);
+      }
+    } while (not_ok);
+  }
 
   not_ok = mpfi_cmp_fr_default(tmp, y);
   mpfi_clear(pi_over_two);
@@ -129,7 +135,7 @@ int mpfi_sin(mpfi_ptr a, mpfi_srcptr b)
 {
   int inexact_left, inexact_right, inexact=0;
   mp_prec_t prec, prec_left, prec_right;
-  mpfr_t tmp, pi_over_2;
+  mpfr_t tmp;
   mpz_t z, zmod4;
   mpz_t quad_left, quad_right;
   int ql_mod4, qr_mod4;
@@ -312,7 +318,7 @@ int mpfi_cos(mpfi_ptr a, mpfi_srcptr b)
 {
   int inexact_left, inexact_right, inexact=0;
   mp_prec_t prec, prec_left, prec_right;
-  mpfr_t tmp, pi_over_2;
+  mpfr_t tmp;
   mpz_t z, zmod4;
   mpz_t quad_left, quad_right;
   int ql_mod4, qr_mod4;
